import datetime
import os

from midiutil.MidiFile import MIDIFile


class MIDI(object):

    def __init__(self, fragment_list, onset, tempo, path):

        # create midi file object
        track_count = self.track_count(fragment_list)
        midi_file = MIDIFile(track_count)
        # add tracks and tempo
        for track in range(track_count):
            midi_file.addTrackName(track, onset, "Track " + str(track))
            midi_file.addTempo(track, onset, tempo)
        # add notes
        note_list = []
        for f in fragment_list:
            note_list += f.flat()
        for n in note_list:
            midi_file.addNote(n.track,
                              n.channel,
                              n.pitch,
                              n.onset,
                              n.duration,
                              n.intensity)
        # open file and write to it
        if path is None:
            # construct default path
            filename = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            filename += ".mid"
            path = os.path.join(os.path.expanduser("~"), filename)
        f = open(path, "wb")
        midi_file.writeFile(f)
        # clean up
        f.close()

    @staticmethod
    def track_count(fragment_list):
        count = 1  # TODO: check for at least one event in one fragment
        for f in fragment_list:
            f_count = f.max_track()
            if f_count > count:
                count = f_count
        return count
