import math

import const


def dynamic_to_midi(dynamic):
    """
    Convert dynamic mark to MIDI velocity.

    :param dynamic: dynamic mark (e.g. "p")
    :type dynamic: str
    :returns: int -- MIDI velocity

    """
    velocity = None
    for key, value in const.ID_MAP.items():
        if value == dynamic:
            velocity = key
            break
    return velocity


def log_seq(lin_seq, start=0.0, duration=1.0):
    """
    Derive a logarithmic sequence from a linear sequence, optionally scaling
    and shifting it.

    :param lin_seq: sequence of consecutive, non-negative, non-zero integers
    :type lin_seq: list
    :param start: start of logarithmic sequence, i.e. shift
    :type start: float
    :param duration: duration of logarithmic sequence, i.e. scale
    :type duration: float
    :returns: list -- logarithmic sequence

    """
    base = len(lin_seq)
    result = []
    for x in lin_seq:
        n = math.log(x, base)
        n *= duration  # scale
        n += start  # shift
        n = round(n, 2)
        result.append(n)
    return result
